#!/usr/bin/env python2.3
#-----------------------------------------------------------------------------
# Name:        bitrate.py
# Purpose:     Some functions to calculate bitrates + command line app.
#
# Author:      Marc 'BlackJack' Rintsch
#
# Created:     2003-11-09
# RCS-ID:      $Id: bitrate.py,v 1.1.1.1 2003/11/18 18:42:19 bj Exp $
# Copyright:   (c) 2003
# Licence:     GPL
#-----------------------------------------------------------------------------
"""Functions to calculate bitrates."""

__version__ = '0.1'
__date__ = '2003-11-18'
__author__ = "Marc 'BlackJack' Rintsch"
__credits__ = 'The authors of transcode for an excellent movie transcoder.'


def bitrateToBytes(bitrate, length):
    """Calculate the number of bytes from bitrate and length.
    
    bitrate -- in kbit/s
    length  -- in seconds
    
    See also: bytesToBitrate()
    """
    return float(bitrate * length * 1000) / 8


def bytesToBitrate(size, length):
    """Calculate the the bitrate from number of bytes and length.
    
    size    -- in bytes
    length  -- in seconds
    
    See also: bitrateToBytes()
    """
    return float(size * 8) / length / 1000


def calcBitrate(targetSize, otherSize, length, containerOverhead=0.0):
    """Calculate the max. bitrate usable for targetSize if there is
    already otherSize worth of data.
    
    targetSize  -- size of the target (e.g. CDs) in bytes
    otherSize   -- size of data (e.g. sound) that has to fit along with
                   the data we want to know the bitrate for, in bytes.
    length      -- length in seconds
    containerOverhead=0.0
                -- take that much percent of overhead from the container
                   format (e.g. AVI, OGM, MKV...) into account
    """
    overhead = 1.0 + (containerOverhead / 100.0)
    size = (targetSize - otherSize) / overhead
    return bytesToBitrate(size, length)


def parseLength(length):
    """Parse a string of format [<h>':'[<m>':']]<s> and return the number
    of seconds as integer.
    
    Hours (<h>), minutes (<m>) and seconds (<s>) are not restricted,
    i.e. '2:198:2' is a legal length (2 hours + 198 minutes + 2 seconds).
    
    Examples:
        parseLength('10')       => 10
        parseLength('725')      => 725
        parseLength('10:05')    => 605
        parseLength('104:03')   => 6243
        parseLength('1:0:0')    => 3600
    """
    result = 0
    multiplier = 1
    hms = length.split(':')
    hms.reverse()
    for x in hms:
        result += int(x) * multiplier
        multiplier *= 60
    return result


def main():
    """Use option -h for usage info."""
    import sys
    import os
    from optparse import OptionParser

    # TODO: catch some exceptions and make user friendly output from them!
    
    # A couple of target sizes in MB.
    sizes = [185, 247, 650, 700, 1300, 1400]

    # TODO: Overhead relative to AVI.
    #overheads = { 'none':0.0, 'avi':1.0, 'ogm':1.5, 'mkv':0.0 }
        
    length = 0      # Length in seconds.
    otherSize = 0   # Size of other material in bytes.
    overhead = 0    # Overhead imposed by the container format.
    
    # TODO: Move option parsing into own (private) function.
    USAGE = '''usage: %prog [options] length [file ...]

The length argument has '[<hours>:]<minutes>:]<seconds>' format. Every
value may "overflow" i.e. it's okay to type ``1:90:0`` which is actually
2 hours and a half.

The given files' sizes are taken into account when calculating the bitrate.'''

    parser = OptionParser(usage=USAGE, version='%prog 0.1')
    parser.add_option('-b', '--audio-bitrate', action='append', type='int',
                      dest='audio_bitrate', default=[],
                      help='Bitrate of audio stream.  Can be given several ' \
                      'times which means "add up all given bitrates".  If ' \
                      'neither this option nor any filename(s) were given, ' \
                      'a value of 128 kbit/s is assumed.')
    parser.add_option('-o', '--container-overhead', type='string',
                      dest='overhead', default=0,
                      help='''Overhead of container format in %.  Default = 0.0.
Every container format has some overhead.  AVI and OGM have a quite high one
while Matroska is a very lean format.  The information on the net about
the values varies a bit.  AVI and OGM need a value between 1.0 and 1.5 here
and Matroska needs 0.5 or even a bit lower.''')
    
    (options, args) = parser.parse_args()

    # If length is not given on command line then print help.
    if not args:
        parser.print_help()
        sys.exit()

    length = parseLength(args[0])
    files = args[1:]
    
    overhead = float(options.overhead)

    # If no bitrate or files where given on the command line
    # assume 128 kbit/s for other data.
    if not options.audio_bitrate and len(args) < 2:
        options.audio_bitrate = [128]

    otherSize = bitrateToBytes(sum(options.audio_bitrate), length) + \
                sum([os.stat(f).st_size for f in files])
    
    print 'Length: %d seconds\n' \
          ' Other: %.2f MB @ %.1f kbit/s' % \
        (length,
         otherSize / float(1024*1024),
         bytesToBitrate(otherSize, length))

    fmt = 'Target:%5d MB => %6.1f MB @ %.1f kbit/s'
    
    def f(size):
        """Build tupel suitable for format string `fmt` above."""
        bytes = size * 1024 * 1024
        return (size,
                float(bytes - otherSize) / (1024*1024),
                calcBitrate(bytes, otherSize, length, overhead))

    print '\n'.join([fmt % t for t in [f(s) for s in sizes] if t[1] > 0])


if __name__ == '__main__':
    main()
